@@ -271,7 +271,7 @@ class TestHelperUtilities:
         assert len(names) == 2
         assert "Series 1" in names
     
-    def test_get_frequencies_from_config(self):
+    def test_get_frequencies(self):
         """Test frequency extraction from config."""
         series_list = [
             SeriesConfig(series_id="S1", frequency="m", transformation="chg", blocks=[DEFAULT_BLOCK_NAME]),
@@ -280,7 +280,7 @@ class TestHelperUtilities:
         blocks = {DEFAULT_BLOCK_NAME: {"factors": 2}}
         config = DFMConfig(series=series_list, blocks=blocks)
         
-        frequencies = get_frequencies_from_config(config)
+        frequencies = get_frequencies(config)
         assert "m" in frequencies
         assert "q" in frequencies
 
